# generic sch parser with callbacks and optional copy
# if a callback returns 1, no copy is done

BEGIN {
	inattr = "";
	inpin = "";
}

function sch_cpy(inhibit)
{
	if ((!sch_copy) || (inhibit == 1))
		return
	print $0
}


/^C / {
	if (context != 0)
		sch_comp_end()
	sym=$7
	context=1
	sch_cpy(sch_comp_start())
	next
}

/^[[]/ { context = 2; sch_cpy(); next }
/^[]]/ { context = 3; sch_cpy(); next }

/^{/ {
	if (context == 1) {
		# there is no [] part of the component - not embedded, need to load it
		error("Component is not embedded!");
		# context = 3;
	}
	else if (last_net) {
		context = 5
	}
	sch_cpy()
	next
}

{ last_net = 0 }

/^N / {
	last_net=1
}


/^}/ {
	if (context == 5) {
		context = 0
		sch_cpy()
	}
	else if (context == 3) {
# end of component
		sch_cpy(sch_comp_end())
		context = 0
		comppin = 0
	}
	else {
# end of pin
		sch_cpy(sch_pin_end())
	}
	inattr = "";
	inpin = "";
	next
}

/^P / {
	if (context != 2)
		error("P (pin) out of symbol context")
	inpin = comppin++
	sch_cpy(sch_pin_start());
	getline
	if ($0 != "{")
		error("P (pin) is not followed by a {} block")
	sch_cpy()
	next
}

($1 ~ "=") {
	key=$0
	val=$0
	sub("=.*$", "", key)
	sub("^[^=]*=", "", val)
	if (inpin == "") {
		if (context == 2) {
			# symbol attrs
			sch_cpy(sch_sym_attr(key, val))
			next
		}
		else if (context == 3) {
			# instance attrs
			sch_cpy(sch_instance_attr(key, val))
			next
		}
		else if (context == 5) {
# ignore attributes of nets and copy transparently
			sch_cpy();
			next
		}
		else
		error("Attribute out of context")
	}
	else {
		sch_cpy(sch_pin_attr(key, val))
	}
	next
}

{ sch_cpy(); }

